# PRTG Custom Sensor to read Homemmatic sensors using XML-API
#
# 20141117 Ver 1.0  Initial Version

param (
   [string]$sensorgroup="bad",      # Group of sensors from the XML File
   [string]$configfile = "C:\Program Files (x86)\PRTG Network Monitor\Custom Sensors\EXEXML\prtg-homematic2.xml"  # optional alternate XML-File
)

write-host "Start prtg-hommatic Version 2.0" (get-date).tostring()

write-host ("Parameter: sensorgroup :" + $sensorgroup)
write-host ("Parameter: configfile  :" + $configfile)

write-host "Initializing PRTG Result variable"
[string]$prtgresult=""
$prtgresult+="<?xml version=""1.0"" encoding=""Windows-1252"" ?>`r`n"
$prtgresult+="<prtg>`r`n"
[bool]$errorfound = $false

if ($sensorgroup -eq "") {
	write-host "Parametercheck: Please specify sensorgroup"
	$prtgresult+="   <error>2</error>"
	$prtgresult+="   <text>Please specify sensorgroup as parameter</text>"
	$errorfound = $True
}
else {

	write-host ("Loading ConfigXML")
	
	try {
		[xml]$configxml = get-content $configfile
		if ($configxml.'prtg-homematic2' -eq $null) {
			write-host "Parametercheck: Please check XML. Rootnode prtg-homematic2 missing"
			$prtgresult+="   <error>2</error>"
			$prtgresult+="   <text>Please check XML. Rootnode prt-homematic2 missing</text>"
			$errorfound = $True
		}
	}
	catch {
		write-host "Parametercheck: unable to load/parse config-XML"
		$prtgresult+="   <error>2</error>"
		$prtgresult+="   <text>unable to load/parse config-XML</text>"
		$errorfound = $True
	}
	
	if (!$errorfound) {
		[xml]$ccudata = $null
		try {
			$cachefile = $configxml.'prtg-homematic2'.cachefile
			if ($cachefile -ne $null) {
				write-host (" Checking  Cache File for Cached ResultXML $cachefile")
				if (test-path -path $cachefile -pathtype leaf) {
					write-host (" Cache-File: found cached ResultXML ")
					if (((get-date) - (get-item -path $cachefile).lastwritetime).totalseconds -lt [int]($configxml.'prtg-homematic2'.cacheage)) {
						write-host (" Cache-File: File is current. Loading data from Cache")
						[xml]$ccudata = get-content -path $cachefile
					}
				}
				else {
					write-host (" Cache-File: Cachefile not found")
				}
			}
			if ($ccudata -eq $null) {
				write-host (" Loading http://"+($configxml.'prtg-homematic2'.ccuip)+"/config/xmlapi/statelist.cgi")
				$webclient = new-object System.Net.WebClient
				[xml]$ccudata = $webclient.DownloadString("http://"+($configxml.'prtg-homematic2'.ccuip)+"/config/xmlapi/statelist.cgi")
				#[xml]$ccudata = $webclient.DownloadString("http://"+$ccuip+"/config/xmlapi/state.cgi?device_id="+$deviceid)
				write-host (" Writing Cache File")
				$ccudata.outerxml | out-file $cachefile
			}
		}
		catch {
			write-host "Unable to retrieve or parse XMLData"
			$prtgresult+="   <error>2</error>`r`n"
			$prtgresult+="   <text>Unable to get XMLData from Cache or CCU - Check log</text>`r`n"
			$errorfound = $true
		}
	}
	
	if (!$errorfound) {
		write-host "Group Filtering for Group: $sensorgroup"
		if (($configxml.SelectSingleNode("//sensor[@name='"+($sensorgroup)+"']").channel) -eq $null) {
			write-host "Group $sensorgroup not found in ConfigXML"
			$prtgresult+="   <error>2</error>`r`n"
			$prtgresult+="   <text>Group not found in ConfigXML Name:$sensorgroup </text>`r`n"
			$errorfound = $true
		}
		else {
			foreach ($sensor in $configxml.SelectSingleNode("//sensor[@name='"+($sensorgroup)+"']").channel) {
				write-host "Progressing ISA_ID:" $sensor.ise_id
				write-host "   Name:"   $sensor.name

				$prtgresult+="    <result>`r`n"
				$prtgresult+="        <channel>"+$sensor.name+"</channel>`r`n"
				$datapoint = $ccudata.SelectSingleNode("//datapoint[@ise_id='"+$sensor.ise_id+"']")

				if ($sensor.typ -eq "") {
					write-host "    Typ = Default"
					$prtgresult+="        <mode>Absolute</mode>`r`n"
				}
				else {
					write-host "    Typ = "$sensor.typ
					$prtgresult+="        <mode>"+$sensor.typ+"</mode>`r`n"
				}
				
				$prtgresult+="        <customunit>"+$datapoint.valueunit+"</customunit>`r`n"
				
				switch ($datapoint.valuetype) {
					"2" {# boolean
						if ($datapoint.value -eq "true") {  
							$prtgresult+="        <value>1</value>`r`n"
						}
						else {  
							$prtgresult+="        <value>0</value>`r`n"
						}
						$prtgresult+="        <float>0</float>`r`n"
						
					}
					"4" {# float
						$prtgresult+="        <value>"+$datapoint.value+"</value>`r`n"
						$prtgresult+="        <float>1</float>`r`n"
					}
					"8" {#integer
						$prtgresult+="        <value>"+$datapoint.value+"</value>`r`n"
						$prtgresult+="        <float>0</float>`r`n"
					}
					"16" {#integer
						$prtgresult+="        <value>"+$datapoint.value+"</value>`r`n"
						$prtgresult+="        <float>0</float>`r`n"
					}
					default {#unknown
						write-host "Unknown valuetype:" $datapoint.valuetype" assuming real"
						$prtgresult+="        <value>"+$datapoint.value+"</value>`r`n"
						$prtgresult+="        <float>1</float>`r`n"
					}
				}
				$prtgresult+="    </result>`r`n"
			}
		}
	}
}

$prtgresult+="</prtg>"
if ($errorfound) {
	write-host "Error Found. Ending with EXIT Code" ([xml]$prtgresult).prtg.error
}
write-host "Sending PRTGRESULT to STDOUT"
$prtgresult

if ($errorfound) {
	exit ([xml]$prtgresult).prtg.error
}